{****************************************************************************}
{*                                                                          *}
{*                                IOFUNCT                                   *}
{*                                                                          *}
{*         Ein- und Ausgaberoutinen fr Turbo-Pascal ab Version 5           *}
{*               von Hannes Geenen und Rolf Sander (1988-1994)              *}
{*                                                                          *}
{****************************************************************************}

{$R-,S-,I+,D-,F-,V+,B-,L+}
{$E+} {$N+} {Coprozessor benutzen, falls vorhanden}

Unit IOFunct;

Interface

Uses
  Dos,
  Crt;

{****************************************************************************}
{***************************     Konstanten     *****************************}
{****************************************************************************}

const
  BackSpace  = #8;     {ASCII-Tastencodes mit besonderer Bedeutung}
  Return     = #13;
  CrtlA      = #1;
  CrtlD      = #4;
  CrtlE      = #5;
  CrtlF      = #6;
  CrtlI      = #9;
  CrtlK      = #11;
  CrtlL      = #12;
  CrtlM      = #13;
  CrtlN      = #14;
  CrtlO      = #15;
  CrtlP      = #16;
  CtrlQ      = #17;
  CrtlS      = #19;
  CrtlV      = #22;
  CrtlW      = #23;
  CrtlX      = #24;
  CrtlY      = #25;
  CrtlZ      = #26;
  Escape     = #27;
  {Extended - Character-Codes: diese Codes werden erst als zweites}
  {Zeichen einer Eingabe bergeben das erste Zeichen ist immer #0}
  ExtMark    = #0;
  Ins        = #82;
  Delete     = #83;
  Left       = #75;   
  Right      = #77;
  PgUp       = #73;
  PgDn       = #81;
  CtrlLeft   = #115;
  CtrlRight  = #116;
  CtrlEnd    = #117;
  Up         = #72;
  Down       = #80;
  Home       = #71;
  EndKey     = #79;
  F1         = #59;
  F2         = #60;
  F3         = #61;
  F4         = #62;
  F5         = #63;
  F6         = #64;
  F7         = #65;
  F8         = #66;
  F9         = #67;
  F10        = #68;
  CrtlF3     = #96;
  CrtlF4     = #97;
  CrtlF5     = #98;
  CrtlF6     = #99;
  CrtlF7     = #100;
  CrtlF8     = #101;
  CrtlF9     = #102;
  CrtlF10    = #103;

  {Bildschirmattribute}
  Normal     = 7;
  Intense    = 15;
  Inverse    = 112;
  BlinkInv   = 240;

  {Rahmen fr PrintBox}
  FrameSingle= 'ڿĳ';       {#218, #191, #192, #217, #196, #179}
  FrameDouble= 'ɻȼͺ';       {#201, #187, #200, #188, #205, #186}
  FrameMixed1= 'ոԾͳ';       {#213, #184, #212, #190, #205, #179}
  FrameMixed2= 'ַӽĺ';       {#214, #183, #211, #189, #196, #186}

  NoAttr = $FF;
  {ReadOnly, Hidden, SysFile, VolumeID, Directory, Archive und AnyFile sind
  in der unit DOS definiert}

{****************************************************************************}
{************************     Typvereinbarungen     *************************}
{****************************************************************************}

Type

  {Muster einer Video-Speicherzelle}
  CellType =
    record
      ch   : Char; {Zeichen}
      attr : Byte  {Attribut}
    End;

  {Muster eines Textbildschirms mit 25 Zeilen und 80 Spalten}
  ScreenTyp      = Array [1..25,1..80] of CellType;
  {Zeiger auf den Anfang eines Bildschirms (z.B. Video-Ram)}
  ScrPtrType     = ^ScreenTyp;

  Str6           = String[6];
  Str12          = String[12];
  Str80          = String[80];
  DirPointer     = ^DirRec;

  DirRec =
    record
      Name : Str12;
      Attr : Byte;
      Time : longint;
      Size : longint;
      Next : Dirpointer;
    End;

{****************************************************************************}
{************************     globale Variablen     *************************}
{****************************************************************************}

Var
  _ScreenPtr : ScrPtrType; {zeigt auf VideoRam: $b000:0 = mono, $b800:0 = Color}
  _CursorBottom : Integer; {Normale Cursorscans, werden durch den Videomodus}
                           {bestimmt und in SetScreenPtr gesetzt}

{****************************************************************************}
{***********************     Funktionsprototypen     ************************}
{****************************************************************************}

Procedure SetScreenPtr;
Procedure PrintBox(xLeft,yTop,xRight,yBottom:Integer;Attrib:Byte;Frame:String);
Procedure PrintAttr(Column,Row,Len:Integer;Attrib:Byte);
Procedure PrintXY(Column:Integer;Row:Integer;WrkString:String;Attrib:Byte);
Procedure SetCursorStyle(ScanTop,ScanBottom:Byte);
Procedure ClearLine(Column,Row,len:Integer;Character:Char;Attrib:Byte);
Function EditString(Column,Row,Len:Integer;Var WrkString:String):Char;
Function UpStr(St:String):String;
Function SelectFile(wildcard,addPath:String;Var nameonly:String):String;

{****************************************************************************}
{*************************     Implementation    ****************************}
{****************************************************************************}

Implementation

{****************************************************************************}
{**************************     SetScreenPtr     ****************************}
{****************************************************************************}

Procedure SetScreenPtr;
Var
  ScreenSegment : Word;
  Regs          : Registers;

Begin
  {Test auf EGA-Karte: Die Funktion $12 des Video-Interupts existiert nur bei}
  {EGA-Karten und ermittelt den Status der EGA-Karte.}
  Regs.bh := $ff;
  Regs.cl := $ff;
  Regs.ah := $12;
  Regs.bl := $10;
  Intr ($10, Regs);

  {Bei Monochrom- oder EGA-Karten eruebrigt sich eine Synchronisation der}
  {Bildschirmausgabe. Die CRT-Variable CheckSnow wird bei Bildschirmausgaben}
  {von den Write-Funktionen ausgewertet. LastMode ist ebenfalls in CRT}
  {definiert und enthaelt den aktuellen Textmodus}
  If (lo(LastMode) = 7) or ( (Regs.bh <= $1) and (Regs.cl <= $0f) ) Then
    CheckSnow := False;
  If (lo(LastMode) = 7) Then
  Begin
    ScreenSegment := $b000; {Monochrom-Modi Video-Ram-Segment bei $b000}
    _CursorBottom := 15;
  End Else Begin {Color-Modi Video-Ram-Segment bei $b800}
    ScreenSegment := $b800;
    _CursorBottom := 8;
  End;

  _ScreenPtr := Ptr(ScreenSegment, $0);    {Setzen des ScreenPointers}
  DirectVideo := True;       {Direkte Bildschirmausgabe erlauben}
End;

{****************************************************************************}
{**************************     PrintBox     ********************************}
{****************************************************************************}

{PrintBox zeichnet einen Rahmen mit den angegebenen Parametern}

Procedure PrintBox(
  xLeft   : Integer;   {Spalte der linken oberen Ecke}
  yTop    : Integer;   {Zeile der linken oberen Ecke}
  xRight  : Integer;   {Spalte der rechten unteren Ecke}
  yBottom : Integer;   {Zeile der rechten unteren Ecke}
  Attrib  : Byte;      {Attribut fuer die Rahmenzeichen}
  Frame   : String);   {Satz der Rahmenzeichen}

Var
  i:     Integer;
  Cell: CellType;

Begin
  Cell.Attr := Attrib;                   {Attribut bertragen}
  Cell.ch := Frame[1];                   {linke obere Ecke}
  _ScreenPtr^[yTop,xLeft] := Cell;
  Cell.ch := Frame[2];                   {rechte obere Ecke}
  _ScreenPtr^[yTop,xRight] := Cell;
  Cell.ch := Frame[3];                   {linke untere Ecke}
  _ScreenPtr^[yBottom,xLeft] := Cell;
  Cell.ch := Frame[4];                   {rechte untere Ecke}
  _ScreenPtr^[yBottom,xRight] := Cell;
  Cell.ch := Frame[5];                   {horizontale Linien}
  For i := xLeft+1 to xRight-1 Do Begin
    _ScreenPtr^[yTop,i] := Cell;
    _ScreenPtr^[yBottom,i] := Cell
  End;
  Cell.ch := Frame[6];                   {vertikale Linien}
  For i := yTop+1 to yBottom-1 Do Begin
    _ScreenPtr^[i,xLeft] := Cell;           {links}
    _ScreenPtr^[i,xRight] := Cell            {rechts}
  End
End;

{****************************************************************************}
{*************************     PrintAttr     ********************************}
{****************************************************************************}

{PrintAttr fuellt den angegebenen Bereich mit einem Attribut}

Procedure PrintAttr(
  Column,            {Anfangsspalte fuer Unterlegung}
  Row,               {Anfangszeile fuer Unterlegung}
  len    : Integer;  {Laenge der Zeile}
  Attrib : Byte);    {Attribut fuer Unterlegung}

Var
  i : Integer;

Begin
  For i := 1 to len Do _ScreenPtr^[Row,Column+i-1].attr := Attrib;
End;

{****************************************************************************}
{*************************     PrintXY     **********************************}
{****************************************************************************}

{PrintXY gibt einen String an der angegebenen Position aus}

Procedure PrintXY(
  Column,              {Anfangsspalte der Ausgabe}
  Row       : Integer; {Anfangszeile der Ausgabe}
  WrkString : String;  {auszugebender String}
  Attrib    : Byte);   {Attribut fuer den String}
Var
  cell : CellType;
  i    : Integer;

Begin
  cell.attr := Attrib;
  For i:=1 to length(WrkString) Do
  Begin
    cell.ch := WrkString[i];
    _ScreenPtr^[Row,Column+i-1] := cell;
  End;
End;

{****************************************************************************}
{*************************     SetCursorStyle     ***************************}
{****************************************************************************}

{SetCursorStyle setzt die Cursorform; Werte > 31 machen den Cursor unsichtbar,}
{kleinere Werte werden entsprechend des Aufloesungsvermoegens des gewaehlten}
{Modus unveraendert uebernommen oder der maximalen Zeilenzahl (Scans) pro}
{Zeichen angepat}

Procedure SetCursorStyle(
  ScanTop    : Byte;  {obere Begrenzung des Cursors}
  ScanBottom : Byte); {untere Begrenzung des Cursors}

Var
  Regs : Registers;

Begin
  Regs.ah := $01;
  Regs.ch := ScanTop;
  Regs.cl := ScanBottom;
  Intr($10,Regs)
End;

{****************************************************************************}
{***************************     ClearLine     ******************************}
{****************************************************************************}

{ClearLine ueberschreibt den angegebenen Bereich mit dem Zeichen}

Procedure ClearLine(
  Column    : Integer; {Anfangsspalte des Bereichs}
  Row       : Integer; {Zeile des Bereichs}
  len       : Integer; {Laenge des Fuellbereichs}
  Character : Char;    {Zeichen fuer die Fuellung}
  Attrib    : Byte);   {Attribut fuer die Fuellung}

Var
  Cell : CellType;
  i    : Integer;

Begin
  cell.ch := Character;
  cell.attr := attrib;
  For i := 1 to len Do
    _ScreenPtr^[Row,Column+i-1] := cell;
End;

{****************************************************************************}
{*************************     EditString     *******************************}
{****************************************************************************}

{EditString zeigt einen String am Bildschirm an und stellt ihn zur}
{Aenderung zur Verfuegung. Der Code der Taste, mit der die Eingabe}
{beendet wurde, wird als Return-Wert dem aufrufenden Programm uebermittelt.}
{Intern verwendet EditString die Sondertasten Insert, Delete,}
{Backspace, Left, Right, Home und End zur Editierung des Strings.}

Function EditString(
  Column,                  {Anfangsspalte des Editbereichs}
  Row,                     {Anfangszeile des Editbereichs}
  Len           : Integer; {Lnge des Editbereiches}
  Var WrkString : String   {EditierString}
  )             : Char;    {Edit-Ende Zeichen}

Var
  ch         : Char;          {Puffer fuer eingegebenes Zeichen}
  count,                      {Zaehlt eingegebene Zeichen}
  pos,                        {Position im String}
  i          : Integer;
  ToggleIns,                  {Schalter fuer Insert/Replace}
  EndOfInput : boolean;       {Flag fuer Eingabeende}
  StartEdit  : Boolean;       {True nur vor dem ersten Tastendruck}

{****************************************************************************}

Function Scan(
  Var Key : Char {Code der gedrueckten Taste}
  ) : boolean;   {True bei Extended-Code, sonst False}

Begin
  Scan := False;
  Key := readkey;
  If Key = ExtMark Then
  Begin
    Key := readkey;
    Scan := True;
  End;
End;

{****************************************************************************}

Begin
  {Vorbereitungen: Platzhalter am am Bildschirm anzeigen}
  ClearLine(Column,Row,Len,'_',Intense);
  PrintXY(Column,Row,Copy(WrkString,1,Len),Inverse); {String anzeigen}
  count := length(WrkString); {Zaehler initialisieren}
  If count > len Then count := len;
  Pos := 1;
  Column := Column-1; {Korrektur, da Pos > 0}
  ToggleIns := False; {Flags initialisieren}
  EndOfInput := False;
  StartEdit := True;
  SetCursorStyle (_CursorBottom-2,_CursorBottom);
  Repeat
    GotoXY(Column+pos,Row); {Cursor positionieren}
    If scan(ch) Then
    Begin {Extended Codes auswerten}
      case ch of
        Ins : {Insert/Replace}
        Begin
          If ToggleIns Then
          Begin
            ToggleIns := False;
            SetCursorStyle (_CursorBottom-2,_CursorBottom);
          End Else Begin
            ToggleIns := True;
            SetCursorStyle(0,_CursorBottom);
          End;
        End;
        Delete : {aktuelles Zeichen loeschen}
        Begin
          For i:= pos to count-1 Do
            _ScreenPtr^[Row,Column+i] := _ScreenPtr^[Row,Column+i+1];
          If count > 0 Then
            count := count-1;
          _ScreenPtr^[Row,Column+count+1].ch := '_';
        End;
        Right : {Cursor nach rechts}
        If (pos <= count) and (pos < len) Then
          pos := pos+1;
        Left : {Cursor nach links}
        If pos > 1 Then
          pos := pos-1;
        Home : {an den Anfang}
          pos := 1;
        EndKey : {ans Ende}
          pos := count+1;
      End; {case}
    End Else Begin    {Auswertung von Standard-Ascii-Sonderzeichen}
      case ch of
        Return,Escape :
          EndOfInput := True;
        BackSpace: {loescht Zeichen}
        Begin
          If pos > 1 Then
          Begin
            pos := pos-1;
            For i := pos to count-1 Do
              _ScreenPtr^[Row,Column+i] := _ScreenPtr^[Row,Column+i+1];
            _ScreenPtr^[Row,Column+count].ch := '_';
            Count := Count-1
          End;
        End;
      Else
        Begin {alle anderen Zeichen werden angezeigt}
          If StartEdit Then {EingabeString lschen}
          Begin
            Count := 0;
            Pos := 1;
            ClearLine(Column+1,Row,len,'_',Intense);
          End;
          If pos <= len Then
          Begin
            If pos > count Then
              count := count+1
            Else Begin
              If ToggleIns Then
              Begin
                If count < len Then count := count+1;
                For i:=count-1 Downto pos Do
                  _ScreenPtr^[Row,Column+i+1] := _ScreenPtr^[Row,Column+i];
              End;
            End;
            _ScreenPtr^[Row,Column+pos].ch := ch;
            Pos := Pos+1;
          End;
        End;
      End; {case}
    End;
    StartEdit := False;
    PrintAttr(Column+1,Row,Len,Intense);
  Until EndOfInput;
  If ch <> Escape Then {Escape = Abbruch}
  Begin
    For i:=1 to count Do {WrkString aktualisieren}
      WrkString[i] := _ScreenPtr^[Row,Column+i].ch;
    WrkString[0] := Char(count)           {Laenge aktualisieren}
  End;
  Count := length(WrkString);                {WrkString anzeigen}
  PrintXY(Column+1,Row,WrkString,Normal);
  ClearLine(Column+count+1,Row,len-count,' ',Normal);
  SetCursorStyle(_CursorBottom-2,_CursorBottom);     {Cursorform normal}
  EditString := ch;
End;

{****************************************************************************}
{*****************************     UpStr     ********************************}
{****************************************************************************}

{converts String to upper case}

Function UpStr(  
  St : String    {InputString}
  )  : String;   {OutputString}
Var
  I : Integer;
Begin
  For I:=1 to length(St) Do St[I] := UpCase(St[I]);
  UpStr := St;
End;

{****************************************************************************}
{***************************     ReadDir     ********************************}
{****************************************************************************}

Procedure ReadDir(
  Path             : String;
  Yes,No           : Byte;
  Var CurrentError : Byte;
  Var Number       : Word;
  Var Start        : DirPointer);

Var
  Entry      : SearchRec;
  EntryNew   : DirPointer;

{****************************************************************************}

Procedure InsertList(
  Var EntryOld : DirPointer;
  EntryNew     : DirPointer);
Var
  P : DirPointer;
Begin
  If EntryOld = Nil Then
  Begin
    EntryOld := EntryNew;
  End Else Begin
    If (EntryOld^.Name > EntryNew^.Name) and (EntryNew^.Name<>'..')
      or (EntryOld^.Name='..') Then
    Begin
      P := EntryOld;
      EntryOld := EntryNew;
      EntryNew^.Next := P;
    End Else Begin
      If EntryOld^.Next = Nil Then
      Begin
        EntryOld^.Next := EntryNew
      End Else Begin
        InsertList(EntryOld^.Next,EntryNew);
      End;
    End;
  End;
End;

{****************************************************************************}

Function Test (Entry : SearchRec) : Boolean;
Begin
  with Entry Do
    Test := (((Attr and Yes) =Yes) or (Yes=NoAttr)) and
            (((Attr and No) =0) or (No=NoAttr)) and
            (name <> '.');
End;

{****************************************************************************}

Begin
  Number := 0;
  FindFirst(Path,anyFile,Entry);
  CurrentError := DosError;
  If (CurrentError=0) and Test(Entry) Then
  Begin
    Inc(Number);
    new(EntryNew);
    with EntryNew^Do
    Begin
      Name := Entry.Name;
      Attr := Entry.Attr;
      Time := Entry.Time;
      Size := Entry.Size;
      Next := Nil;
    End;
    InsertList(Start,EntryNew);
  End;
  While CurrentError = 0 Do
  Begin
    FindNext(Entry);
    CurrentError := DosError;
    If (CurrentError = 0) and Test(Entry) Then
    Begin
      Inc(Number);
      new(EntryNew);
      with EntryNew^Do
      Begin
        Name := Entry.Name;
        Attr := Entry.Attr;
        Time := Entry.Time;
        Size := Entry.Size;
        Next := Nil;
      End;
      InsertList(Start,EntryNew);
    End;
  End;
End;

{****************************************************************************}
{*************************     SelectDirRec     *****************************}
{****************************************************************************}

Function SelectDirRec(
  Title   : Str80;
  Start   : DirPointer
  )       : DirPointer;

Var
  Ch1                       : Char;
  Ch2                       : Char;
  Column                    : Byte;
  DirWindow                 : Byte;
  MaxDirWindow              : Byte;
  Number                    : Integer;
  OldWindMax                : Word;
  OldWindMin                : Word;
  P                         : DirPointer;
  PointerList               : array [0..3,1..8] of DirPointer;
  Row                       : Byte;
  St                        : String[14];
  TextAttrOld               : Byte;
  TheEnd                    : Boolean;
  xPos                      : Byte;
  yPos                      : Byte;

{****************************************************************************}

Procedure WindowPaint(
  Title  : Str80);

Const
  xLeft   =  8;
  xRight  = 73;
  yBottom = 12;
  yTop    =  3;

Begin
  PrintBox(xLeft,yTop,xRight,yBottom,Inverse,FrameMixed1);
  window(1,1,80,25);
  If xRight-xLeft > length(Title) + 4 Then
    PrintXY(xLeft+(xRight-xLeft-Length(Title)) div 2
      ,yTop,Title,Inverse);
  window(xLeft+1,yTop+1,xRight-1,yBottom-1);
  clrscr;
End;

{****************************************************************************}

{Expandiert kurze Filenamen auf 8 Characters}

Function Expand(
  Name : String
  )    : Str12;

Var
  a,i : Integer;
  s   : String;

Begin
  a := pos('.',Name);
  If a > 1 Then
  Begin
    s := '';
    For i :=a to 8 Do s := s + ' ';
    insert(s,Name,a);
  End;
  Expand := Name;
End;

{****************************************************************************}

Procedure RedrawScreen;
Var
  I    : Byte;
  s    : Byte;
  z    : Byte;
  Test : Boolean;

Begin
  For s:=0 to 3 Do
    For Z :=1 to 8 Do
      PointerList[s,z] := nil;
  TextAttr := 112;
  P := Start;
  S := 0;
  Z := 1;
  clrscr;
  For I := 1 to DirWindow*4 Do P:= P^.Next;
  Repeat
    gotoxy(3+s*16,z);
    PointerList[s,z] := P;
    If (P^.Attr and Directory)=directory
      Then write(P^.Name,'\')
    Else
      write(expand(P^.Name));
    Test := (P^.Next =Nil) or ((z=8) and (s=3));
    If Not(Test) Then
    Begin
      P:=P^.Next;
      Inc(s);
      If s=4 Then
      Begin
        s:=0;
        Inc(z);
      End;
    End;
  Until test;
End;

{****************************************************************************}

Begin
  SelectDirRec := Nil;
  If Start=Nil Then Exit;
  TextAttrOld := TextAttr;
  Textattr    := 112;
  OldWindMax := WindMax;
  OldWindMin := WindMin;
  xPos := wherex;
  yPos := wherey;
  WindowPaint(' '+Title+' ');
  Row := 1;
  Column := 0;
  DirWindow := 0;
  P := Start;
  Number :=1;
  While P^.Next <> Nil Do
  Begin
    Inc(Number);
    P:=P^.Next;
  End;
  Number := Number-32;
  If Number<1 Then
    MaxDirWindow :=0
  Else Begin
    MaxDirWindow := Number div 4;
    If Number mod 4>0 Then Inc(MaxDirWindow);
  End;
  RedrawScreen;
  TheEnd := false;
  Repeat
    gotoxy(2+Column*16,Row);
    Textattr :=15;
    with PointerList[Column,Row]^ Do
      If (Attr and Directory) = Directory Then
        St := ' '+Name+'\'
      Else
        St := ' '+expand(Name);
    write(St,'':14-length(St));
    Ch1 := readkey;
    case ch1 of
      Escape :
      Begin
        SelectDirRec := Nil;
        TheEnd := true;
      End;
      Return :
      Begin
        SelectDirRec := PointerList[Column,Row];
        TheEnd := true;
      End;
      ExtMark :
      Begin
        Ch2 := readkey;
        If (Ch2 <>PgUp) and (Ch2<>PgDn) Then
        Begin
          gotoxy(2+Column*16,Row);
          TextAttr := 112;
          with PointerList[Column,Row]^Do
            If (Attr and Directory) = Directory Then
              St := ' '+Name+'\'
            Else
              St := ' '+expand(Name);
          write(St,'':14-length(St));
        End;
        case Ch2 of
          Home :
          Begin
            Row := 1;
            Column := 0;
            DirWindow := 0;
            RedrawScreen;
          End;
          EndKey :
          Begin
            DirWindow := MaxDirWindow;
            Row := 1;
            Column := 0;
            RedrawScreen;
            While (Row<8) and (PointerList[Column,Row+1]<>Nil)
              Do Inc(Row);
            While (Column<3) and (PointerList[Column+1,Row]<>Nil)
              Do Inc(Column);
          End;
          Up :
          If Row>1 Then
            Dec(Row)
          Else
            If DirWindow > 0 Then
            Begin
              Dec(DirWindow);
              RedrawScreen;
            End;
          Down :
          If Row<8 Then
          Begin
            If PointerList[Column,Row+1] <> Nil Then Inc(Row)
          End Else
            If DirWindow<MaxDirWindow Then
            Begin
              Inc(DirWindow);
              RedrawScreen;
              While PointerList[Column,Row]=Nil Do Dec(Column);
            End;
          Left :
          If Column>0 Then
            Dec(Column)
          Else
            If Row>1 Then
            Begin
              Dec(Row);
              Column := 3;
            End Else
              If DirWindow>0 Then
                 Begin
                   Dec(DirWindow);
                   Row := 8;
                   Column := 3;
                   RedrawScreen;
                 End;
          Right :
          If Column<3 Then
          Begin
            If PointerList[Column+1,Row] <> Nil Then Inc(Column);
          End Else
            If Row<8 Then
            Begin
              If PointerList[0,Row+1] <> Nil Then
              Begin
                Inc(Row);
                Column := 0;
              End
            End Else
              If DirWindow < MaxDirWindow Then
              Begin
                Inc(DirWindow);
                Column := 0;
                RedrawScreen;
              End;
          PgUp :
          Begin
            If DirWindow>7 Then
            Begin
              Dec(DirWindow,8);
              RedrawScreen;
            End Else Begin
              DirWindow := 0;
              RedrawScreen;
            End;
          End;
          PgDn :
          Begin
            If DirWindow<(MaxDirWindow-8) Then
            Begin
              Inc(DirWindow,8);
              RedrawScreen;
              While PointerList[Column,Row]=Nil Do Dec(Column);
            End Else Begin
              DirWindow := MaxDirWindow;
              RedrawScreen;
              While PointerList[Column,Row]=Nil Do Dec(Column);
            End;
          End;
        End;{case}
      End;
    Else {any other key typed}
      Begin
        If (Ord(ch1)>=97) and (Ord(ch1)<=122) Then {convert to upper case}
          Ch1 := Chr(Ord(ch1)-32); 
        If (Ord(ch1)>=65) and (Ord(ch1)<=90) Then
        Begin
          If DiskSize(Ord(ch1)-64)>0 Then
          Begin
            SelectDirRec       := PointerList[Column,Row];
            SelectDirRec^.Name := 'NewDrive = '+ch1;
            TheEnd := true;
          End;
        End;
      End;
    End;{case}
  Until TheEnd;
  window(lo(OldWindMin)+1,hi(OldWindMin)+1,lo(OldWindMax)+1,hi(OldWindMax)+1);
  gotoxy(xPos,yPos);
  TextAttr := TextAttrOld;
End;

{****************************************************************************}
{**************************      SelectFile      ****************************}
{****************************************************************************}

Function SelectFile(
  wildcard     : String;
  addPath      : String;
  Var nameonly : String
  )            : String;

Var
  Start          : DirPointer;
  CurrentFile    : DirPointer;
  CurrentError   : Byte;
  sp             : Byte;
  Number         : Word;
  Path           : String;
  directoryflag  : boolean;

{****************************************************************************}

Procedure FreeDir(
  Var DP : DirPointer);

Begin
  If DP <> Nil Then
  Begin
    If DP^.Next <> Nil Then FreeDir(DP^.Next);
    dispose(DP);
  End;
End;

{****************************************************************************}

Begin
  Start := Nil;
  GetDir(0,Path);
  Path := Path + AddPath;
  Repeat
    freedir(Start);
    Start := Nil;
    ReadDir(Path+'\'+wildcard,NoAttr,SysFile+Hidden+VolumeID+Directory,
      CurrentError,Number,Start);
    ReadDir(Path+'\*.*',Directory,NoAttr,
      CurrentError,Number,Start);
    CurrentFile := Selectdirrec(Path+'\'+wildcard,Start);
    directoryflag := (CurrentFile^.attr and Directory) <> 0;
    Path := Path + '\' + CurrentFile^.name;
    nameonly := CurrentFile^.name;
    If (CurrentFile^.name = '..') Then
    Begin
      sp := length(Path)-3;
      Path := copy(Path,1,sp);
      While Path[sp] <> '\' Do Dec(sp);
      Path := copy(Path,1,sp-1);
    End;
    If Copy(CurrentFile^.name,1,11) = 'NewDrive = ' Then
    Begin
      GetDir(Ord(CurrentFile^.name[12])-64,Path); {set new drive}
      If Path[Length(Path)] = '\' Then
        Path := Copy(Path,1,Length(Path)-1); {'\' am Ende entfernen}
      Directoryflag := True; {forces Repeat to continue}
    End;
  Until (not Directoryflag) or (CurrentFile=nil);
  If CurrentFile <> nil
    Then SelectFile := Path
  Else
    SelectFile := '';
End;

End.

{****************************************************************************}
{****************************     end of file    ****************************}
{****************************************************************************}
