{****************************************************************************}
{*                                                                          *}
{*                                 MAGIC-3D                                 *}
{*                                                                          *}
{*                Programm zur Erzeugung von Autostereogrammen              *}
{*              Version 1.0 (c) Rolf Sander & Martin Simeth 1994            *}
{*                                                                          *}
{****************************************************************************}

{$R-} {kein range-check}
{$S-} {kein stack-check}
{$I-} {kein I/O-check}
{$D-} {keine Debug-Information}
{$V-} {relaxed var-string-check}
{$B-} {short-circuit boolean evaluation}
{$E+} {Coprozessor emulieren, falls nicht vorhanden}
{$N+} {Coprozessor benutzen}

program MAGIC3D;

uses
  dos,      {fr FindFirst}
  crt,      {fr ClrScr, ReadKey}
  graph,    {fr PutPixel etc.}
  IOFunct;  {Input- und Outputfunktionen}

const
  DepthMax        : Real    = 0.25; {maximale Tiefe, relativ zum Muster}
  DepthInfo       : String  = 'TIEFE.PCX';      {Tiefenbild}
  DepthFileName   : String  = 'TIEFE\TIEFE.PCX';
  DepthxSizeMax             = 3700;             {maximale Breite}
  Footer                    = '   Zurck ins Hauptmen mit <Esc>       '+
                              '                                        ';
  Header                    = '   MAGIC-3D      Version 1.0            '+
                              ' (c) Rolf Sander & Martin Simeth 1994   ';
  Invert          : Boolean = False;            {Tiefe invertieren?}
  InvertInfo      : String  = 'normal';         {Invers oder normal}
  OutputFilename  : String  = 'MAGIC';          {ohne Suffix}
  OutputInfo      : String  = 'Monitor';        {Monitor oder Datei}
  OutputSelect    : Integer = 1;                {Nummer des Ausgabemediums}
  PathToDriver              = 'C:\MAGIC-3D';    {Pfad des BGI-Grafiktreibers}
  PatternFileName : String  = 'MUSTER\MUSTER.PCX';
  PatternInfo     : String  = 'MUSTER.PCX';     {Muster}
  ReplaceFile     : Boolean = false;

type
  PatternIndexType = array[0..DepthxSizeMax-1] of Word;
  MagicLineType    = array[0..DepthxSizeMax] of Byte;
  GreyPaletteType  = array[0..255] of Byte; {Graustufenpalette von Tiefe}
  ColorPaletteType = array[0..255,1..3] of Byte; {Farbpalette von PCX-Muster}
  FileOfCharType   = File Of Char; {Bug in Pascal! Typ ohne Leerzeichen}

Var
  ColorPalette        : ColorPaletteType; {vom MusterFile}
  DefaultTest         : SearchRec;        {Test, ob default existiert}
  Depth               : MagicLineType;    {Tiefenwerte fr eine Zeile}
  DepthBitPerPixel    : Integer;          {Bits pro Pixel im Tiefen PCX-File}
  DepthBytesPerLine   : Integer;          {Bytes pro Zeile im Tiefen PCX-File}
  DepthID             : FileOfCharType;   {ID des Tiefenfiles}
  DepthLineBytes      : Integer;          {Bytes pro Zeile fr alle Planes}
  DepthNPlanes        : Integer;          {Farbplanes im Tiefen PCX-File}
  DepthxSize          : Integer;          {Breite des Tiefenbildes}
  DepthySize          : Integer;          {Hhe des Tiefenbildes}
  ExitMenu            : Boolean;          {Taste Return gedrckt?}
  GreyPalette         : GreyPaletteType;  {fr Bildgenerierung}
  I                   : Integer;          {Laufvariable}
  Key                 : Char;             {gedrckte Taste}
  MagicLine           : MagicLineType;    {berechnete Werte fr eine Zeile}
  MenuLeftRow         : Integer;          {linke Menspalte}
  MenuLines           : Integer;          {Anzahl Menpunkte}
  MenuRows            : Integer;          {Lnge einer Menzeile}
  MenuSelect          : Integer;          {selektierter Menpunkt}
  MenuTopLine         : Integer;          {oberste Menzeile}
  OutputID            : FileOfCharType;   {ID des Magicfiles}
  PatternBitPerPixel  : Integer;          {Bits pro Pixel im Muster PCX-File}
  PatternBytesPerLine : Integer;          {Bytes pro Zeile und Farbebene}
  PatternID           : FileOfCharType;   {ID des Musterfiles}
  PatternIndex        : PatternIndexType; {aktuelle x-Stelle im Muster}
  PatternLine         : MagicLineType;    {fr MusterFile}
  PatternLineBytes    : Integer;          {Bytes pro Zeile im PCX Muster-File}
  PatternNPlanes      : Integer;          {Zahl der Farbebenen in PCX-Muster}
  PatternxSize        : Integer;          {x-Gre des Musterfiles}
  PatternySize        : Integer;          {y-Gre des Musterfiles}
  PixelLine           : MagicLineType;    {zum PCX einlesen}
  x                   : Word;             {aktuelle x-Position}
  y                   : Word;             {aktuelle y-Position}
Label
  Abort; {Programmabbruch, wenn default nicht vorhanden}

{****************************************************************************}
{**********************      ShowInfoText       *****************************}
{****************************************************************************}

procedure ShowInfoText;
Begin
  ClrScr;
  PrintXY(1,1,Copy(Header,1,80),inverse);
  Writeln;
  Writeln;
  Writeln('  Das Programm MAGIC-3D erzeugt Autostereogramme. Hierzu werden zwei');
  Writeln('  Eingabebilder im PCX-Format bentigt:');
  Writeln;
  Writeln('  1. Ein Muster, das fr das vordergrndige Erscheinungsbild des');
  Writeln('     Autostereogramms sorgt. Diese Datei sollte im Verzeichnis MUSTER liegen.');
  Writeln;
  Writeln('  2. Ein Bild, das die im Autostereogramm versteckte Tiefeninformation enthlt.');
  Writeln('     Die Graustufe jedes Pixels entspricht seiner spteren Tiefe (je dunkler,');
  Writeln('     desto tiefer). Diese Tiefenbilddatei sollte im Verzeichnis TIEFE liegen.');
  Writeln;
  Writeln('  Es empfiehlt sich, zunchst Autostereogramme mit den mitgelieferten');
  Writeln('  Beispieldateien (z.B. MUSTER.PCX und TIEFE.PCX) zu erzeugen. Eigene');
  Writeln('  Bilder knnen Sie mit jedem Zeichenprogramm malen, das im');
  Writeln('  PCX-Format abspeichern kann.');
  Writeln;
  Writeln('  Das erzeugte Bild kann wahlweise auf dem Monitor ausgegeben werden oder');
  Writeln('  als PCX-Datei im Verzeichnis MAGIC abgespeichert werden.');
  Writeln;
  Writeln('  Durch die Cursortasten oder durch Eingabe des Kennbuchstabens in der');
  Writeln('  Klammer knnen Sie einen Menpunkt selektieren. Mit der Returntaste');
  Writeln('  wird der entsprechende  Menpunkt ausgefhrt.');
  ClearLine(1,25,80,' ',Inverse);
  PrintXY(1,25,'   Zurck ins Hauptmen mit <Esc>',Inverse);
  Repeat
    Key := Readkey;
  Until Key=Escape;
End;

{****************************************************************************}
{********************        InvertDepth        *****************************}
{****************************************************************************}

procedure InvertDepth;
Begin
  Invert := not Invert;
  If Invert Then
    InvertInfo := 'invertiert'
  Else
    InvertInfo := 'normal';
End;

{****************************************************************************}
{*************************    SelectDepth    ********************************}
{****************************************************************************}

procedure SelectDepth;
Var
  OldDepthInfo     : String;
  OldDepthFileName : String;
Begin
  ClrScr;
  PrintXY(17,15,'Bitte Tiefendatei auswhlen und <Return> drcken!',normal);
  PrintXY(32,17,'   <Esc> = Abbruch',normal);
  PrintXY(25,19,'<A>..<Z> = Wechsel des Laufwerks',normal);
  OldDepthInfo     := DepthInfo;
  OldDepthFileName := DepthFileName;
  DepthFileName := SelectFile('*.PCX','\TIEFE',DepthInfo);
  If DepthFileName = '' Then Key := Escape;
  If Key=Escape Then
  Begin
    DepthInfo     := OldDepthInfo;
    DepthFileName := OldDepthFileName;
  End;
End;

{****************************************************************************}
{************************    SelectPattern     ******************************}
{****************************************************************************}

procedure SelectPattern;
Var
  OldPatternInfo     : String;
  OldPatternFileName : String;
Begin
  ClrScr;
  PrintXY(17,15,'Bitte Musterdatei auswhlen und <Return> drcken!',normal);
  PrintXY(32,17,'   <Esc> = Abbruch',normal);
  PrintXY(25,19,'<A>..<Z> = Wechsel des Laufwerks',normal);
  OldPatternInfo     := PatternInfo;
  OldPatternFileName := PatternFileName;
  PatternFileName := SelectFile('*.PCX','\MUSTER',PatternInfo);
  If PatternFileName = '' Then Key := Escape;
  If Key=Escape Then
  Begin
    PatternInfo     := OldPatternInfo;
    PatternFileName := OldPatternFileName;
  End;
End;

{****************************************************************************}
{*************************    SelectOutput    *******************************}
{****************************************************************************}

procedure SelectOutput;
Var
  Answer            : Char;
  OldOutputSelect   : Integer;
  OldOutputInfo     : String;
  OldOutputFileName : String;
Begin
  ClrScr;
  OldOutputSelect   := OutputSelect;
  OldOutputInfo     := OutputInfo;
  OldOutputFileName := OutputFileName;
  OutputSelect      := 0; {dieses Men wird mit Datei als default gestartet}
  PrintXY(1,1,Copy(Header,1,80),inverse);
  PrintXY(8,15,'<D>atei..........als PCX-Datei speichern',normal);
  PrintXY(8,17,'<M>onitor........Bildausgabe auf dem Monitor',normal);
  PrintXY(1,25,footer,inverse);
  PrintAttr(7,OutputSelect*2+15,68,inverse);
  ExitMenu := false; {Taste <Return> nicht gedrckt}
  Repeat
    Key := ReadKey;
    PrintAttr(7,OutputSelect*2+15,68,normal);
    Case UpCase(Key) of
      'D'     : OutputSelect := 0;
      'M'     : OutputSelect := 1;
      Escape  : ExitMenu := True;
      return  : ExitMenu := True;
      extmark :
      Begin
        Key := ReadKey;
        Case Key of
          EndKey : OutputSelect := 1;
          Home   : OutputSelect := 0;
          Up     : OutputSelect := (OutputSelect+1) mod 2;
          Down   : OutputSelect := (OutputSelect+1) mod 2;
        End;{Case}
      End;
    End;{Case}
    PrintAttr(7,OutputSelect*2+15,68,inverse);
  Until ExitMenu;
  If not (Key=Escape) Then Case OutputSelect of
    0 :
    Begin
      ExitMenu := false;
      Repeat
        PrintBox(7,7,74,13,Normal,Framedouble);
        PrintXY(23,9,'Ausgabefile auswhlen:',Normal);
        PrintXY(54,9,'.PCX',Normal);
        Key := EditString(46,9,8,OutputFileName);
        OutputFileName := UpStr(OutputFileName);
        If (Key=Return) Then
        Begin
          Assign(OutputID,'MAGIC\' + Outputfilename + '.PCX');
          Reset(OutputID);
          If IOResult=0 Then
          Begin
            Close(OutputID);
            PrintXY(12,11,
              'vorhandene Datei berschreiben? <Return>=ja, <Esc>=nein',
              Normal);
            Repeat
              Answer := ReadKey;
            Until (Answer=Escape) Or (Answer=Return);
            If Answer=Return Then
              ExitMenu := True;
              ReplaceFile := True;
            ClearLine(10,11,62,' ',normal);
          End Else Begin
            Rewrite(OutputID);
            If IOResult=0 Then
            Begin
              Close(OutputID);
              ExitMenu := True;
            End Else Begin
              PrintXY(19,11,
                'dieser Dateiname ist ungltig, bitte ndern!',normal);
            End;
          End;
        End;
      Until (Key=Escape) Or (ExitMenu);
      OutputInfo := 'File: ' + OutputFilename + '.PCX';
    End;
    1 : OutputInfo := 'Monitor';
  End;{Case}
  If Key=Escape Then
  Begin
    OutputSelect   := OldOutputSelect;
    OutputInfo     := OldOutputInfo;
    OutputFileName := OldOutputFileName;
  End Else Begin
    ReplaceFile := True;
  End;
End;

{****************************************************************************}
{*********************          OpenInput             ***********************}
{****************************************************************************}

{ Diese Routine ffnet ein PCX-File und bestimmt seine Eckwerte. Die         }
{ Eingabe, ob es sich um ein Tiefenbild oder ein Muster handelt, geschieht   }
{ ber die Boolsche Variable DepthInput. Die ID der bereits assign'ten Datei }
{ wird ebenfalls bergeben und in dieser Routine mit PCXID angesprochen. Die }
{ Eckwerte der PCX-Grafik werden unter Pattern/Depth -BitPerPixel,           }
{ -BytesPerLine, -NPlanes, -xSize und -ySize abgelegt.                       }

Procedure OpenInput(
  DepthInput : Boolean;         {True=Tiefe, False=Muster}
  Var PCXID  : FileOfCharType); {File-ID des PCX-Files}
Type
  BigPaletteType = array[0..3*256-1] of Byte;
Var
  BigPalette      : BigPaletteType;
  BitPerPixel     : Integer;
  BytesPerLine    : Integer;
  Character       : Char;
  GreyValue       : Byte;
  I               : Integer;
  J               : Integer;
  NPlanes         : Integer;
  ReadByte        : Byte;
  xMax            : Word;
  xMin            : Word;
  yMax            : Word;
  yMin            : Word;

Begin
  Reset(PCXID);
  Seek(PCXID,3);
  Read(PCXID,Character);
  BitPerPixel := Ord(Character);
  If DepthInput Then
    DepthBitPerPixel := BitPerPixel
  Else
    PatternBitPerPixel := BitPerPixel;

  Read(PCXID,Character);
  xMin := Ord(Character);
  Read(PCXID,Character);
  xMin := xMin+Ord(Character)*256;

  Read(PCXID,Character);
  yMin := Ord(Character);
  Read(PCXID,Character);
  yMin := yMin+Ord(Character)*256;

  Read(PCXID,Character);
  xMax := Ord(Character);
  Read(PCXID,Character);
  xMax := xMax+Ord(Character)*256;

  Read(PCXID,Character);
  yMax := Ord(Character);
  Read(PCXID,Character);
  yMax := yMax+Ord(Character)*256;

  Seek(PCXID,16);
  If DepthInput Then
  Begin
    For I := 0 To 15 Do
    Begin
      GreyValue := 0;
      For J := 1 To 3 Do
      Begin
        Read(PCXID,Character);
        GreyValue := GreyValue + Round(Ord(Character)/3);
      End;
      GreyPalette[i] := GreyValue;
    End;
  End Else Begin
    For I := 0 To 15 Do
    Begin
      For J := 1 To 3 Do
      Begin
        Read(PCXID,Character);
        ColorPalette[i,j] := Ord(Character);
      End;
    End;
  End;

  Seek(PCXID,65);
  Read(PCXID,Character);
  NPlanes := Ord(Character);
  If DepthInput Then
    DepthNPlanes := NPlanes
  Else
    PatternNPlanes := NPlanes;

  Read(PCXID,Character);
  BytesPerLine := Ord(Character);
  Read(PCXID,Character);
  BytesPerLine := BytesPerLine+Ord(Character)*256;
  If DepthInput Then
    DepthBytesPerLine := BytesPerLine
  Else
    PatternBytesPerLine := BytesPerLine;

  If BitPerPixel=8 Then {groe Palette am Dateiende lesen}
  Begin
    Seek(PCXID,Filesize(PCXID)-768);
    For I := 0 to 768-1 Do
    Begin
      Read(PCXID,Character);
      BigPalette[I] := Ord(Character);
    End;
    If DepthInput Then  {Fr Tiefenbild Graustufen berechnen}
    Begin
      For I := 0 to 255 Do
      Begin
        GreyPalette[i] := 
          Round((BigPalette[3*I]+BigPalette[3*I+1]+BigPalette[3*I+2])/3);
      End;
    End Else Begin {Fr Muster-file ColorPalette bestimmen}
      For I := 0 to 255 Do
      Begin
        ColorPalette[i,1] := BigPalette[3*I];
        ColorPalette[i,2] := BigPalette[3*I+1];
        ColorPalette[i,3] := BigPalette[3*I+2];
      End;
    End;
  End;

  Seek(PCXID,128);
  If (NPlanes=1) And (BitPerPixel=1) Then {schwarz/wei}
  Begin
    If DepthInput Then
    Begin
      GreyPalette[0] := 0;
      GreyPalette[1] := 128;
    End Else Begin
      ColorPalette[0,1] := 0;    { schwarz }
      ColorPalette[0,2] := 0;
      ColorPalette[0,3] := 0;
      ColorPalette[1,1] := 255;  { wei }
      ColorPalette[1,2] := 255;
      ColorPalette[1,3] := 255;
      ColorPalette[15,1] := 255; { letzte Farbe auch wei fr OutTextXY }
      ColorPalette[15,2] := 255;
      ColorPalette[15,3] := 255;
    End;
  End;

  If DepthInput Then
  Begin
    DepthxSize       := xMax-xMin+1;
    DepthySize       := yMax-yMin+1;
    DepthLineBytes   := NPlanes*BytesPerLine;
  End Else Begin
    PatternxSize     := xMax-xMin+1;
    PatternySize     := yMax-yMin+1;
    PatternLineBytes := NPlanes*BytesPerLine;
  End;
End;

{****************************************************************************}
{*********************          ReadPCXLine              ********************}
{****************************************************************************}

{ ReadPCXLine liest wahlweise eine Bildzeile vom MusterFile oder vom         }
{ TiefenFile. Die ID der entsprechenden Datei wird einfach bergeben.        }
{ Rckgabe ist das array PixelLine, in dem die eingelesenen Werte stehen,    }
{ entweder zu interpretieren als Farbe, oder als Tiefeninformation.          }

Procedure ReadPCXLine(
  DepthInput : Boolean;         {True=Tiefe, False=Muster}
  Var PCXID  : FileOfCharType); {File-ID des PCX-Files}
Var
  Bit          : Byte;
  BitPerPixel  : Integer;
  BytesInLine  : Integer;
  BytesPerLine : Integer;
  Character    : Char;
  Duplicate    : Integer;
  I            : Integer;
  J            : Integer;
  LineBytes    : Integer;
  NPlanes      : Integer;
  PixelValue   : Byte;
  PCXLine      : MagicLineType;
  ReadByte     : Byte;
  ReadInteger  : Integer;
  Total        : Integer;

Begin
  Total := 0;
  If DepthInput Then
  Begin
    LineBytes    := DepthLineBytes;
    BytesPerLine := DepthBytesPerLine;
    BitPerPixel  := DepthBitPerPixel;
    NPlanes      := DepthNPlanes;
  end else begin
    LineBytes    := PatternLineBytes;
    BytesPerLine := PatternBytesPerLine;
    BitPerPixel  := PatternBitPerPixel;
    NPlanes      := PatternNPlanes;
  End;

  While Total<=LineBytes-1 Do
  Begin
    Read(PCXID,Character);
    ReadInteger := Ord(Character);
    If ReadInteger>=192 Then
    Begin
      Duplicate := ReadInteger-192;
      Read(PCXID,Character);
      ReadByte := Ord(Character);
      For I := 1 To Duplicate Do
      Begin
        PCXLine[Total] := ReadByte;
        Total := Total+1;
      End;
    End Else Begin
      PCXLine[Total] := ReadInteger;
      Total := Total+1;
    End;
  End;

  If BitPerPixel=1 Then
  Begin
  For BytesInLine := 0 To BytesPerLine-1 Do
    Begin
      For I := 0 to 7 Do
      Begin
        Bit := 128 Shr I;
        PixelValue := 0;
        For J := 0 To NPlanes-1 Do
        Begin
          PixelValue := PixelValue +
            (((PCXLine[BytesInLine+J*BytesPerLine]) And Bit) Shr (7-I)) *
            (1 shl J);
        End;
        PixelLine[(BytesInLine shl 3)+I] := PixelValue;
      End;
    End;
  End;

  If BitPerPixel=8 Then
  Begin
    For BytesInLine := 0 To BytesPerLine-1 Do
    Begin
      PixelLine[BytesInLine] := PCXLine[BytesInLine];
    End;
  End;
End;

{****************************************************************************}
{*********************           ReadDepthLine            *******************}
{****************************************************************************}

{berechnet die Tiefe fr alle x-Werte in der aktuellen Zeile y.}

procedure ReadDepthLine;
Begin
  ReadPCXLine(True,DepthID);
  For x := 0 To DepthxSize-1 Do
  Begin
    Depth[x] := GreyPalette[PixelLine[x]]; {0 <= Depth <= 255}
    If Invert Then Depth[x] := 255 - Depth[x];
  End;
End;

{****************************************************************************}
{*********************        ReadPatternLine      **************************}
{****************************************************************************}

Procedure ReadPatternLine;
Begin
  If (y mod PatternySize)=0 Then
    Seek(PatternID,128);
  ReadPCXLine(false,PatternID);
  For x := 0 To PatternxSize-1 do
  Begin
    PatternIndex[x] := x;
    MagicLine[x] := PixelLine[x];
  End;
End;

{****************************************************************************}
{*********************        InitScreen              ***********************}
{****************************************************************************}


Procedure InitScreen;
Var
  Blue                   : Byte;
  Color                  : Integer;
  Graphdriver            : Integer;
  Graphmode              : Integer;
  Green                  : Byte;
  Palette                : PaletteType;
  Red                    : Byte;

Begin
  GraphDriver := 0;
  GraphMode := 0;
  InitGraph(GraphDriver,Graphmode,PathToDriver);
  SetViewPort(0,0,GetMaxx,GetMaxy,True);
  ClearViewPort;
  SetLineStyle(0,0,1);

  For I := 0 To 15 Do
  Begin
    Color := 0;
    Red := ColorPalette[i,1];
    Red := Red Shr 6;
    Case Red of
      1: Color := Color Or 32;
      2: Color := Color Or 4;
      3: Color := Color Or 36;
    End;
    Green := ColorPalette[i,2];
    Green := Green Shr 6;
    Case Green of
      1: Color := Color Or 16;
      2: Color := Color Or 2;
      3: Color := Color Or 18;
    End;
    Blue := ColorPalette[i,3];
    Blue := Blue Shr 6;
    Case Blue of
      1: Color := Color Or 8;
      2: Color := Color Or 1;
      3: Color := Color Or 9;
    End;
    Palette.Colors[i] := Color;
    Palette.Size := 16
  End;
  SetAllPalette(Palette);
  SetTextJustify(0,0);
  OutTextXY(0,GetMaxY,'Abbruch durch <Esc>');

End;

{****************************************************************************}
{*********************          OpenOutput           ************************}
{****************************************************************************}

Procedure OpenOutput;
Var
  I         : Integer;
  Character : Char;
  Text      : String;

Begin
  Assign(OutputID,'MAGIC\' + Outputfilename + '.PCX');
  Rewrite(OutputID);
  Character := Char(10);
  Write(OutputID,Character);
  Character := Char(5);
  Write(OutputID,Character);
  Character := Char(1);
  Write(OutputID,Character);
  Character := Char(8); {BitPerPixel}
  Write(OutputID,Character);
  Character := Char(0);
  Write(OutputID,Character);
  Write(OutputID,Character);
  Write(OutputID,Character);
  Write(OutputID,Character);
  Character := Char((DepthxSize-1) And 255);
  Write(OutputID,Character);
  Character := Char((DepthxSize-1) Shr 8);
  Write(OutputID,Character);
  Character := Char((DepthySize-1) And 255);
  Write(OutputID,Character);
  Character := Char((DepthySize-1) Shr 8);
  Write(OutputID,Character);
  Character := Char(DepthxSize And 255);
  Write(OutputID,Character);
  Character := Char(DepthxSize Shr 8);
  Write(OutputID,Character);
  Character := Char(DepthySize And 255);
  Write(OutputID,Character);
  Character := Char(DepthySize Shr 8);
  Write(OutputID,Character);

  For I := 0 To 15 Do
  Begin
    Write(OutputID,Char(Colorpalette[i,1]));
    Write(OutputID,Char(Colorpalette[i,2]));
    Write(OutputID,Char(Colorpalette[i,3]));
  End;
  Character := Char(0);
  Write(OutputID,Character);
  Character := Char(1); {NPlanes}
  Write(OutputID,Character);
  DepthxSize := (DepthxSize Or 1)-1;
  Character := Char(DepthxSize And 255); {BytesPerLine And 255}
  Write(OutputID,Character);
  Character := Char(DepthxSize Shr 8); {BytesPerLine Shr 8}
  Write(OutputID,Character);
  Character := Char(1);
  Write(OutputID,Character);
  Character := Char(0);
  Write(OutputID,Character);
  Text := 'MAGIC-3D Version 1.0  (c) Rolf Sander & Martin Simeth 1994';
  For I := 1 to 58 Do
  Begin
    Write(OutputID, Text[i]);
  End;

  ClrScr;
  PrintBox(20,10,60,18,Normal,Framedouble);
  Printxy(22,12,'   MAGIC-3D erzeugt ' + Outputfilename + '.PCX',normal);
  Printxy(22,14,'       Prozent',normal);
  Printxy(22,16,'   Abbruch durch <Esc>',normal);

End;

{****************************************************************************}
{*********************           WritePCXLine        ************************}
{****************************************************************************}

Procedure WritePCXLine;
Var
  Duplicate : Integer;
  Character : Char;
  Percent   : String;   {fr busy-Balken}
  Value     : Byte;
  x         : Word;

Begin
  x := 0;
  while x<=DepthxSize-1 Do
  Begin
    Duplicate := 0;
    Value := MagicLine[x+Duplicate];
    Duplicate := Duplicate+1;
    While (Value=MagicLine[Duplicate+x])
      And (Duplicate<63) And ((x+Duplicate)<=(DepthxSize-1)) Do
    Begin
      Duplicate := Duplicate+1;
    End;
    If (Duplicate>1) Or (Value>=192) Then
    Begin
      Character := Char(192+Duplicate);
      Write(OutputID,Character);
    End;
    Write(OutputID, Char(Value));
    x := x+Duplicate;
  End;
  Str(Round(y*100./DepthySize),Percent);
  Printxy(25,14,Percent,normal);
End;

{****************************************************************************}
{*********************          CloseOutput          ************************}
{****************************************************************************}

Procedure CloseOutput;
Var
  Key : Char;

Begin
  Key := Char(12);
  Write(OutputID,Key);
  For I := 0 To 255 Do
  Begin
    Write(OutputID,Char(Colorpalette[i,1]));
    Write(OutputID,Char(Colorpalette[i,2]));
    Write(OutputID,Char(Colorpalette[i,3]));
  End;
  Close(OutputID);
  ReplaceFile := False;
End;

{****************************************************************************}
{******************          ForwardAlgorithm             *******************}
{****************************************************************************}

procedure ForwardAlgorithm;
Var
  OldDepth : Integer;
  NewDepth : Integer;
  Index    : Integer;
Begin
  OldDepth := PatternxSize;
  For x := 0 To DepthxSize-1 do
  Begin
    NewDepth := Round(PatternxSize*(1-DepthMax*Depth[x]/255));
    MagicLine[x+NewDepth] := MagicLine[x];
    PatternIndex[x+NewDepth] := PatternIndex[x];
    If (OldDepth<NewDepth) Then
    Begin
      For I := 1 To (NewDepth-OldDepth) Do
      Begin
        Index := ((PatternIndex[x]-i+PatternxSize) mod PatternxSize);
        MagicLine[x+NewDepth-i] := MagicLine[Index];
        PatternIndex[x+NewDepth-i] := Index;
      End;
    End;
    OldDepth := NewDepth;
  End;
End;

{****************************************************************************}
{*********************          CreateMagic           ***********************}
{****************************************************************************}

procedure CreateMagic;
Var
  Answer : Char;
Label
  TheEnd,
  EndOfSbr;
Begin
  ClrScr;
  FindFirst(PatternFilename,AnyFile,DefaultTest);
  If DosError<>0 Then
  Begin
    Writeln('Datei '+PatternFilename+' kann nicht geffnet werden!');
    Key := ReadKey;
    Goto EndOfSbr;
  End;
  FindFirst(DepthFilename,AnyFile,DefaultTest);
  If DosError<>0 Then
  Begin
    Writeln('Datei '+DepthFilename+' kann nicht geffnet werden!');
    Key := ReadKey;
    Goto EndOfSbr;
  End;
  If OutputSelect=0 Then {nur bei Ausgabe in PCX-Datei}
  Begin
    Assign(OutputID,'MAGIC\' + Outputfilename + '.PCX');
    Reset(OutputID);
    If IOResult=0 Then {nur, wenn PCX-File existiert}
    Begin
      Close(OutputID);
      If not ReplaceFile Then {nur, wenn PCX-File gerade erst erzeugt}
      Begin
        PrintBox(7,7,74,13,Normal,Framedouble);
        PrintXY(12,9,'vorhandene Datei ' + Outputfilename +
          '.PCX berschreiben?',Normal);
        PrintXY(12,11,'<Return>=ja, <Esc>=nein',Normal);
        Repeat
          Answer := ReadKey;
        Until (Answer=Escape) Or (Answer=Return);
        If Answer=Escape Then {nur, wenn <Esc> gedrckt wurde}
          Goto EndOfSbr;
      End;
    End;
  End;
  Assign(DepthID,DepthFileName);
  OpenInput(True, DepthID);
  Assign(PatternID,PatternFileName);
  OpenInput(false, PatternID);
  Case OutputSelect of
    0 : OpenOutput;
    1 : InitScreen;
  End;{Case}
  For y := 0 to DepthySize-1 do
  Begin
    ReadDepthLine;
    ReadPatternLine;
    ForwardAlgorithm;
    Case OutputSelect of
      0 : WritePCXLine;
      1 :
      Begin
        For x := 0 To DepthxSize-1 do
        Begin
          PutPixel(x,y,MagicLine[x]);
        End;
      End;
    End;{Case}
    If KeyPressed Then
      If ReadKey=Escape Then
        Goto TheEnd;
  End;
  If OutputSelect=1 Then Repeat
    Key := ReadKey;
  Until (Key=Escape);
  TheEnd:
  Close(DepthID);
  Close(PatternID);
  Case OutputSelect of
    0 : CloseOutput;
    1 : CloseGraph;
  End;{Case}
  EndOfSbr:
End;

{****************************************************************************}
{*****************************       main       *****************************}
{****************************************************************************}

Begin
  FindFirst('MUSTER',Directory,DefaultTest);
  If DosError<>0 Then
  Begin
    Writeln('Fataler Fehler: Verzeichnis MUSTER ist nicht vorhanden!');
    Key := ReadKey;
    Goto Abort;
  End;
  FindFirst('TIEFE',Directory,DefaultTest);
  If DosError<>0 Then
  Begin
    Writeln('Fataler Fehler: Verzeichnis TIEFE ist nicht vorhanden!');
    Key := ReadKey;
    Goto Abort;
  End;
  FindFirst('MAGIC',Directory,DefaultTest);
  If DosError<>0 Then
  Begin
    Writeln('Fataler Fehler: Verzeichnis MAGIC ist nicht vorhanden!');
    Key := ReadKey;
    Goto Abort;
  End;

  Randomize;
  SetScreenPtr; {notwendig fr PrintBox etc.}

  MenuSelect  := 0;
  MenuLines   := 7;
  MenuTopLine := 8;
  MenuRows    := 68;
  MenuLeftRow := 7;

  Repeat
    ClrScr;
    PrintXY(1,1,Header,inverse);
    PrintBox(MenuLeftRow-3,MenuTopLine-5,
      MenuLeftRow+MenuRows+2,MenuTopLine+2*MenuLines,normal,framedouble);
    PrintXY(MenuLeftRow, MenuTopLine-3,
      ' Gewnschten Menpunkt selektieren und <Return> drcken!',normal);
    PrintXY(MenuLeftRow,MenuTopLine   ,
      ' <H>ilfe.......Informationen zu diesem Programm',normal);
    PrintXY(MenuLeftRow,MenuTopLine+2,
      ' <T>iefe.......Tiefenbild aussuchen ('+DepthInfo+')',normal);
    PrintXY(MenuLeftRow,MenuTopLine+4,
      ' <M>uster......Muster aussuchen ('+PatternInfo+')',normal);
    PrintXY(MenuLeftRow,MenuTopLine+6,
      ' <B>erechnen...Magic-3D Bild Berechnung starten',normal);
    PrintXY(MenuLeftRow,MenuTopLine+8,
      ' <S>peichern...Ausgabemedium bestimmen ('+OutputInfo+')',normal);
    PrintXY(MenuLeftRow,MenuTopLine+10,
      ' <I>nvers......Hhen- und Tiefendarstellung ist '+InvertInfo,normal);
    PrintXY(MenuLeftRow,MenuTopLine+12,
      ' <Q>uit........Programmabbruch',normal);
    ClearLine(1,25,80,' ',inverse);

    ExitMenu := false; {Taste <Return> nicht gedrckt}
    Repeat
      SetCursorStyle(32,32); {Cursor unsichtbar}
      PrintAttr(MenuLeftRow,MenuSelect*2+MenuTopLine,MenuRows,inverse);
      Key := ReadKey;
      PrintAttr(MenuLeftRow,MenuSelect*2+MenuTopLine,MenuRows,normal);
      Case UpCase(Key) of
        'H'     : MenuSelect := 0;
        'T'     : MenuSelect := 1;
        'M'     : MenuSelect := 2;
        'B'     : MenuSelect := 3;
        'S'     : MenuSelect := 4;
        'I'     : MenuSelect := 5;
        'Q'     : MenuSelect := 6;
        return  : ExitMenu := True;
        extmark :
        Begin
          Key := ReadKey;
          Case Key of
            F1   :
            Begin
              MenuSelect := 0;
              ExitMenu := True;
            End;
            EndKey : MenuSelect := MenuLines-1;
            Home   : MenuSelect := 0;
            Up     : MenuSelect := (MenuSelect+MenuLines-1) mod MenuLines;
            Down   : MenuSelect := (MenuSelect+1) mod MenuLines;
          End;{Case}
        End;
      End;{Case}
    Until ExitMenu;
    Case MenuSelect of
      0 : ShowInfoText;
      1 : SelectDepth;
      2 : SelectPattern;
      3 : CreateMagic;
      4 : SelectOutput;
      5 : InvertDepth;
    End;{Case}
  Until MenuSelect=6;
  SetCursorStyle(30,31); {Cursor normal}
  ClrScr;
  Abort:
End. {main}

{****************************************************************************}
{****************************     end of file    ****************************}
{****************************************************************************}
